import { DislikeFilled, LikeFilled, RightOutlined } from "@ant-design/icons";
import "./App.css";
import { Badge, Flex, Result, Skeleton, Space, Tag } from "antd";
import { useEffect, useRef, useState } from "react";
import { motion } from "framer-motion";

function SkeletonPlaceholder() {
  return (
    <>
      <Skeleton.Input active={true} size="large" block={true} />
      <br />
      <Skeleton.Input active={true} size="large" block={true} />
      <br />
      <Skeleton.Input active={true} size="large" block={true} />
      <br />
      <Skeleton.Input active={true} size="large" block={true} />
    </>
  );
}

function ShowMoreButton({ onClick }) {
  return (
    <Flex justify="center">
      <button className="show-more-button" onClick={onClick}>
        Alle anzeigen
      </button>
    </Flex>
  );
}

function UseVoteLocalStorage() {
  const getVotes = () => {
    const votes = localStorage.getItem("shx-product-pipeline-votes");
    return votes === null ? [] : JSON.parse(votes);
  };

  const [storageVotes, setStorageVotes] = useState(getVotes());

  const saveVotes = (votes) => {
    setStorageVotes(votes);
    localStorage.setItem("shx-product-pipeline-votes", JSON.stringify(votes));
  };

  const vote = (id, up) => {
    const newVotes = [...storageVotes];

    const existingVoteIndex = newVotes.findIndex((vote) => vote.id === id);

    if (existingVoteIndex !== -1) {
      newVotes[existingVoteIndex].t = up === true ? 1 : 0;
    } else {
      newVotes.push({
        id: id,
        t: up === true ? 1 : 0,
      });
    }

    saveVotes(newVotes);
  };

  return {
    votes: storageVotes,
    vote: vote,
    setVotes: saveVotes,
  };
}

function Card({
  rankingPosition,
  name,
  productVariant,
  productCharacteristics,
  rightComponent,
}) {
  const MiddleComponent = () => {
    return (
      <Flex vertical style={{ textAlign: "left" }}>
        <span>{name}</span>
        {productVariant !== undefined &&
          productCharacteristics !== undefined && (
            <span style={{ fontSize: 16, color: "#535c68" }}>
              {productVariant}: {productCharacteristics}
            </span>
          )}
      </Flex>
    );
  };

  return (
    <div className="card">
      <Flex justify="space-between">
        {rankingPosition === undefined ? (
          <div style={{ paddingLeft: 10 }}>
            <MiddleComponent />
          </div>
        ) : (
          <>
            <Flex gap={4} style={{ textAlign: "left" }} align="center">
              <span style={{ color: "#787878", fontSize: 18 }}>
                #{rankingPosition}
              </span>
            </Flex>
            <div style={{ width: "100%", textAlign: "left", marginLeft: 10 }}>
              <MiddleComponent />
            </div>
          </>
        )}
        <Flex vertical justify="center">
          {rightComponent}
        </Flex>
      </Flex>
    </div>
  );
}

function VoteRequest(id, up) {
  fetch(`${process.env.REACT_APP_API_URL}/vote?t=${up === true ? "u" : "d"}`, {
    method: "POST",
    headers: {
      "Content-Type": "application/json",
    },
    body: JSON.stringify({
      id: id,
    }),
  }).catch(() => {});
}

// Create a function to compare date values
/*
function comparePublishedAt(a, b) {
  // Check if PublishedAt for both a and b is empty
  if (!a.PublishedAt && !b.PublishedAt) {
    return 0; // If both are empty, they are considered equal
  }
  if (!a.PublishedAt) {
    return 1; // If a is empty but b is not, a is considered greater
  }
  if (!b.PublishedAt) {
    return -1; // If b is empty but a is not, b is considered greater
  }

  // Convert the published date values to Date objects
  const dateA = new Date(a.PublishedAt.split(".").reverse().join("."));
  const dateB = new Date(b.PublishedAt.split(".").reverse().join("."));

  // Sort by the latest date
  if (dateA > dateB) {
    return -1;
  }
  if (dateA < dateB) {
    return 1;
  }
  return 0;
} */

function dateDiffInDays(a, b) {
  const diffInMs = Math.abs(b - a);
  return Math.ceil(diffInMs / (1000 * 60 * 60 * 24));
}

function App() {
  const { votes, vote, setVotes } = UseVoteLocalStorage();
  const votesRef = useRef(votes);
  const pageHeightRef = useRef(0);

  useEffect(() => {
    votesRef.current = votes;

    sendPageHeightToIframe();
  }, [votes]);

  const [products, setProducts] = useState({
    NewProducts: [],
    InWorkProducts: [],
    FutureProducts: [],
  });

  const [showMore, setShowMore] = useState({
    newProducts: false,
    inWorkProducts: false,
    futureProducts: false,
  });

  // update page height on click on show more button
  useEffect(() => sendPageHeightToIframe(), [showMore]);

  useEffect(() => {
    const fetchProducts = () =>
      fetch(process.env.REACT_APP_API_URL)
        .then((res) => res.json())
        .then((data) => {
          setProducts(data);

          // remove votes for products which no longer in future products
          setVotes(
            votesRef.current.filter((v) =>
              data.FutureProducts.some((item) => item.Id === v.id)
            )
          );
        })
        .catch(() => {});

    fetchProducts();

    setInterval(() => fetchProducts(), 5000);
  }, []);

  // send the height to the iframe
  function sendPageHeightToIframe() {
    const height = document.body.scrollHeight;

    if (pageHeightRef.current === height) return;

    pageHeightRef.current = height;
    window.parent.postMessage(height, "*");
  }

  const currentDate = new Date();

  return (
    <div className="app">
      <div className="container">
        <span className="subtitle">Auf dem Laufenden bleiben</span>
        <h1 className="header">Unsere neuesten Produkte</h1>

        <div className="cards-container" style={{ paddingBottom: 40 }}>
          <div className="cards">
            {products.NewProducts === null ? (
              <Result status={404} title="Aktuell nichts neues :(" />
            ) : products.NewProducts.length > 0 ? (
              <>
                {products.NewProducts /*.sort(comparePublishedAt) */.map(
                  (product, index) =>
                    (showMore.newProducts || index < 5) && (
                      <Card
                        key={index}
                        name={
                          <Space>
                            {product.Name}
                            {dateDiffInDays(
                              currentDate,
                              new Date(
                                product.PublishedAt.split(".")
                                  .reverse()
                                  .join(".")
                              )
                            ) < 3 && (
                              <Tag style={{ fontSize: 14 }} color="#af9363">
                                Neu
                              </Tag>
                            )}
                          </Space>
                        }
                        productVariant={product.Variant}
                        productCharacteristics={product.Characteristics}
                        rightComponent={
                          product.Url === "" ? (
                            <></>
                          ) : (
                            <RightOutlined
                              style={{ fontSize: 18 }}
                              onClick={() => window.open(product.Url)}
                            />
                          )
                        }
                      />
                    )
                )}

                {products.NewProducts.length > 5 && !showMore.newProducts && (
                  <ShowMoreButton
                    onClick={() =>
                      setShowMore((sM) => {
                        return { ...sM, newProducts: true };
                      })
                    }
                  />
                )}
              </>
            ) : (
              <SkeletonPlaceholder />
            )}
          </div>
        </div>

        <span className="subtitle">Bald bei uns erhältlich</span>
        <h1 className="header">Aktuell in Arbeit</h1>

        <div className="cards-container" style={{ paddingBottom: 40 }}>
          <div className="cards">
            {products.InWorkProducts === null ? (
              <Result
                status={404}
                title="Aktuell nichts in Arbeit"
                subTitle="Wird Zeit etwas zu ändern. Jetzt Produktwunsch äußern!"
              />
            ) : products.InWorkProducts.length > 0 ? (
              <>
                {products.InWorkProducts.map(
                  (product, index) =>
                    (showMore.inWorkProducts || index < 5) && (
                      <Card
                        key={index}
                        name={product.Name}
                        productVariant={product.Variant}
                        productCharacteristics={product.Characteristics}
                        rightComponent={<Badge status="processing" />}
                      />
                    )
                )}

                {products.InWorkProducts.length > 5 &&
                  !showMore.inWorkProducts && (
                    <ShowMoreButton
                      onClick={() =>
                        setShowMore((sM) => {
                          return { ...sM, inWorkProducts: true };
                        })
                      }
                    />
                  )}
              </>
            ) : (
              <SkeletonPlaceholder />
            )}
          </div>
        </div>

        <span className="subtitle">Jetzt abstimmen</span>
        <h1 className="header">Zukünftige Produkte</h1>
        <Flex justify="center" style={{ marginTop: 4 }}>
          <span style={{ color: "#474747", maxWidth: 800 }}>
            Gestalte unser Sortiment mit! Hier hast du die Möglichkeit, für das
            nächste Produkt abzustimmen und die Reihenfolge zukünftiger Produkte
            mit deiner Stimme zu beeinflussen. Deine Meinung zählt!
          </span>
        </Flex>

        <div className="cards-container">
          <div className="cards" style={{ paddingBottom: 20 }}>
            {products.FutureProducts === null ? (
              <Result
                status={404}
                title="Aktuell liegen keine Produktwünsche vor"
                subTitle="Jetzt Produktwunsch äußern!"
              />
            ) : products.FutureProducts.length > 0 ? (
              <>
                {products.FutureProducts.sort((a, b) => b.Votes - a.Votes).map(
                  (product, index) =>
                    (showMore.futureProducts || index < 5) && (
                      <motion.div
                        key={product.Id}
                        layout
                        initial={{ opacity: 0 }}
                        animate={{ opacity: 1 }}
                        exit={{ opacity: 0 }}
                        transition={{ duration: 0.5 }}
                      >
                        <Card
                          name={product.Name}
                          rankingPosition={index + 1}
                          productVariant={product.Variant}
                          productCharacteristics={product.Characteristics}
                          rightComponent={
                            <div style={{ display: "flex", gap: 12 }}>
                              <DislikeFilled
                                style={{
                                  color:
                                    votes.findIndex(
                                      (vote) =>
                                        vote.id === product.Id && vote.t === 0
                                    ) > -1
                                      ? "#e74c3c"
                                      : "#34495e",
                                }}
                                onClick={() => {
                                  if (
                                    votes.findIndex(
                                      (vote) =>
                                        vote.id === product.Id && vote.t === 0
                                    ) === -1
                                  ) {
                                    VoteRequest(product.Id, false);
                                    vote(product.Id, false);

                                    // simulate vote before request is made
                                    setProducts((products) => {
                                      const newArr =
                                        products.FutureProducts.map((p) =>
                                          p.Id === product.Id
                                            ? { ...p, Votes: p.Votes - 1 }
                                            : p
                                        );

                                      return {
                                        ...products,
                                        FutureProducts: newArr,
                                      };
                                    });
                                  }
                                }}
                              />
                              <span>{product.Votes}</span>
                              <LikeFilled
                                style={{
                                  color:
                                    votes.findIndex(
                                      (vote) =>
                                        vote.id === product.Id && vote.t === 1
                                    ) > -1
                                      ? "#27ae60"
                                      : "#34495e",
                                }}
                                onClick={() => {
                                  if (
                                    votes.findIndex(
                                      (vote) =>
                                        vote.id === product.Id && vote.t === 1
                                    ) === -1
                                  ) {
                                    VoteRequest(product.Id, true);
                                    vote(product.Id, true);

                                    // simulate vote before request is made
                                    setProducts((products) => {
                                      const newArr =
                                        products.FutureProducts.map((p) =>
                                          p.Id === product.Id
                                            ? { ...p, Votes: p.Votes + 1 }
                                            : p
                                        );

                                      return {
                                        ...products,
                                        FutureProducts: newArr,
                                      };
                                    });
                                  }
                                }}
                              />
                            </div>
                          }
                        />
                      </motion.div>
                    )
                )}

                {products.FutureProducts.length > 5 &&
                  !showMore.futureProducts && (
                    <ShowMoreButton
                      onClick={() =>
                        setShowMore((sM) => {
                          return { ...sM, futureProducts: true };
                        })
                      }
                    />
                  )}
              </>
            ) : (
              <SkeletonPlaceholder />
            )}
          </div>
        </div>
      </div>
    </div>
  );
}

export default App;
